#!/usr/bin/env python3

from os import path, remove, environ, chmod

try:
	from os import uname
except ImportError:
	def uname(): return [_, environ['COMPUTERNAME']]

from yaml import load, dump

from time import sleep

from shutil import copyfile

from paramiko.ssh_exception import SSHException

from colortext import bgre, tabd, error, fatal

from system import userfind

from net.ssh import SecureSHell

from secrecy.gpg import GPGTool

from secrecy.yubi import ykchalres

class PassCrypt(GPGTool, SecureSHell):
	dbg = None
	aal = None
	sho = None
	try:
		user = userfind()
		home = userfind(user, 'home')
	except FileNotFoundError:
		user = environ['USERNAME']
		home = path.join(environ['HOMEDRIVE'], environ['HOMEPATH'])
	plain = path.join(home, '.pwd.yaml')
	crypt = path.join(home, '.passcrypt')
	recvs = []
	remote = ''
	reuser = user
	if 'GPGKEYS' in environ.keys():
		recvs = environ['GPGKEYS'].split(' ')
	elif 'GPGKEY' in environ.keys():
		recvs = recvs + [
            environ['GPGKEY']] if not environ['GPGKEY'] in recvs else []
	def __init__(self, *args, **kwargs):
		for arg in args:
			if hasattr(self, arg):
				setattr(self, arg, True)
		for (key, val) in kwargs.items():
			if hasattr(self, key):
				setattr(self, key, val)
		if self.dbg:
			print(bgre(PassCrypt.__mro__))
			print(bgre(tabd(PassCrypt.__dict__, 2)))
			print(' ', bgre(self.__init__))
			print(bgre(tabd(self.__dict__, 4)))
		GPGTool.__init__(self, *args, **kwargs)
		SecureSHell.__init__(self, *args, **kwargs)
		if self.remote:
			self._copynews_()
		__weaks = self._readcrypt()
		if path.exists(self.crypt) and __weaks is None:
			self._garr()
			__weaks = self._readcrypt()
		try:
			with open(self.plain, 'r') as pfh:
				__newpws = load(pfh.read())
			if not self.dbg:
				remove(self.plain)
		except FileNotFoundError:
			__newpws = {}
		if __newpws:
			__weaks = __weaks if __weaks else {}
			for (k, v) in __newpws.items():
				__weaks[k] = v
		if __weaks != self._readcrypt():
			self._writecrypt(__weaks)
		self.__weaks = __weaks

	def _copynews_(self):
		if self.dbg:
			print(bgre(self._copynews_))
		if self.remote:
			try:
				self.scpcompstats(
                    self.crypt, path.basename(self.crypt),
                    self.remote, self.reuser)
			except (FileNotFoundError, SSHException):
				pass

	def _chkcrypt(self):
		if self.dbg:
			print(bgre(self._chkcrypt))
		if self._readcrypt() == self.__weaks:
			return True

	def _findentry(self, pattern, weakz=None):
		if self.dbg:
			print(bgre(tabd({self._findentry: {'pattern': pattern}})))
		__weakz = weakz if weakz else self.__weaks
		for (u, p) in __weakz.items():
			if pattern == u or (
                  len(p) == 2 and len(pattern) > 1 and pattern in p[1]):
				return p

	def _readcrypt(self):
		if self.dbg:
			print(bgre(self._readcrypt))
		try:
			with open(self.crypt, 'r') as vlt:
				crypt = vlt.read()
		except FileNotFoundError:
			return None
		return load(str(self.decrypt(crypt)))

	def _writecrypt(self, plain):
		if self.dbg:
			print(bgre(tabd({self._writecrypt: {'plain': plain}})))
		kwargs = {'output': self.crypt}
		if self.recvs:
			kwargs['recipients'] = self.recvs
		self.__weaks = plain
		try:
			copyfile(self.crypt, '%s.1'%self.crypt)
			chmod('%s.1'%self.crypt, 0o600)
		except FileNotFoundError:
			pass
		while True:
			self.encrypt(message=dump(self.__weaks), **kwargs)
			if self._chkcrypt():
				if self.remote:
					self._copynews_()
				chmod(self.crypt, 0o600)
				break
		return True

	def adpw(self, usr, pwd=None):
		if self.dbg:
			print(bgre(tabd({
                self.adpw: {'user': self.user, 'entry': usr, 'pwd': pwd}})))
		pwdcom = [pwd if pwd else self._passwd()]
		com = input('enter a comment: ')
		if com:
			pwdcom.append(com)
		if self.dbg:
			print('%s\n adduser = %s addpass = %s'%(
                self.adpw, usr, pwd))
		__weak = self._readcrypt()
		if __weak and self.user in __weak.keys():
			__weak[self.user][usr] = pwdcom
			self._writecrypt(__weak)
			return True

	def chpw(self, usr, pwd=None):
		if self.dbg:
			print(bgre(tabd({
                self.chpw: {'user': self.user, 'entry': usr, 'pwd': pwd}})))
		pwdcom = [pwd if pwd else self._passwd()]
		com = input('enter a comment: ')
		if com:
			pwdcom.append(com)
		if self.dbg:
			print('%s\n adduser = %s addpass, comment = %s'%(
                self.chpw, usr, pwdcom))
		__weak = self._readcrypt()
		if __weak and self.user in __weak.keys() and \
              usr in __weak[self.user].keys():
			__weak[self.user][usr] = pwdcom
			self._writecrypt(__weak)
			return True

	def rmpw(self, usr):
		if self.dbg:
			print(bgre(tabd({self.rmpw: {'user': self.user, 'entry': usr}})))
		__weak = self._readcrypt()
		if __weak and self.user in __weak.keys() and \
              usr in __weak[self.user].keys():
			del __weak[self.user][usr]
			if self._writecrypt(__weak):
				if self.remote:
					self._copynews_()
			return True

	def lspw(self, usr=None, aal=None, display=None):
		if self.dbg:
			print(bgre(tabd({self.lspw: {'user': self.user, 'entry': usr}})))
		aal = True if aal else self.aal
		sho = True if display else self.sho
		if self.__weaks:
			if aal:
				__ents = self.__weaks
				if usr:
					for (user, entrys) in self.__weaks.items():
						__match = self._findentry(usr, self.__weaks[user])
						if __match:
							__ents = {usr: __match}
							break
			elif self.user in self.__weaks.keys():
				__ents = self.__weaks[self.user]
				if usr:
					__ents = {usr: self._findentry(usr, __ents)}
			return __ents

def lscrypt(usr):
	if self.dbg:
		print(bgre(lscrypt))
	if usr:
		return PassCrypt().lspw(usr)

if __name__ == '__main__':
    exit(1)



#/usr/bin/env python3
"""pwclip init module"""
import sys
from os.path import abspath, dirname, exists
# this only makes sence while i need the lib folder in the PYTHONPATH
# otherwise i need to rewrite lots of code cause i have thus libs in the
# python environment path at my workstation and do not change that =)
__lib = '%s/lib'%abspath(dirname(__file__))
if exists(__lib) and __lib not in sys.path:
	sys.path = [__lib] + sys.path
from colortext import abort, fatal
from pwclip.cmdline import cli

def pwclip():
	"""pwclip wrapper function"""
	try:
		cli()
	except RuntimeError as err:
		fatal(err)
	except KeyboardInterrupt:
		abort()


#!/usr/bin/env python3
"""ssh connection and remote command """

#global imports"""
import os
import sys
import time
from io import StringIO
from paramiko import ssh_exception, SSHClient, AutoAddPolicy, SSHException
from shutil import copy2
from socket import \
    getfqdn as fqdn, gaierror as NameResolveError, timeout as sockettimeout

from colortext import bgre, tabd, abort, error, fatal
from system import whoami
from net import askdns

# default vars
__version__ = '0.1'

class SecureSHell(object):
	dbg = None
	reuser = ''
	remote = ''
	def __init__(self, *args, **kwargs):
		for arg in args:
			if hasattr(self, arg):
				setattr(self, arg, True)
		for (key, val) in kwargs.items():
			if hasattr(self, key):
				setattr(self, key, val)
		if self.dbg:
			print(bgre(SecureSHell.__mro__))
			print(bgre(tabd(self.__dict__, 2)))

	def _ssh_(self, remote, reuser=None, port=22):
		if '@' in remote:
			_reuser, remote = remote.split('@')
		reuser = whoami() if not reuser else reuser
		if self.dbg:
			print(bgre('%s\n  remote = %s\n  reuser = %s\n  port = %d'%(
                self._ssh_, remote, reuser, port)))
		ssh = SSHClient()
		ssh.set_missing_host_key_policy(AutoAddPolicy())
		try:
			ssh.connect(
                askdns(remote), int(port),
                username=reuser, allow_agent=True, look_for_keys=True)
		except (ssh_exception.SSHException, NameResolveError) as err:
			error(self._ssh_, err)
			raise err
		return ssh

	def rrun(self, cmd, remote=None, reuser=None):
		remote = remote if remote else self.remote
		#print(remote)
		reuser = reuser if reuser else self.reuser
		if self.dbg:
			print(bgre(self.stdo))
			print(bgre('  %s %s %s'%(reuser, remote, cmd)))
		ssh = self._ssh_(remote, reuser)
		try:
			ssh.exec_command(cmd)
		except (AttributeError, ssh_exception.SSHException) as err:
			error(self.rrun, err)
			raise err

	def rcall(self, cmd, remote=None, reuser=None):
		remote = remote if remote else self.remote
		reuser = reuser if reuser else self.reuser
		#print(remote)
		#print(reuser)
		if self.dbg:
			print(bgre(self.rcall))
			print(bgre('  %s %s %s'%(reuser, remote, cmd)))
		ssh = self._ssh_(remote, reuser)
		try:
			chn = ssh.get_transport().open_session()
			chn.settimeout(10800)
			chn.exec_command(cmd)
			while not chn.exit_status_ready():
				if chn.recv_ready():
					och = chn.recv(1024)
					while och:
						sys.stdout.write(och.decode())
						och = chn.recv(1024)
				if chn.recv_stderr_ready():
					ech = chn.recv_stderr(1024)
					while ech:
						sys.stderr.write(ech.decode())
						ech = chn.recv_stderr(1024)
			return int(chn.recv_exit_status())
		except (
            AttributeError, ssh_exception.SSHException, sockettimeout
            ) as err:
			error(self.rcall, err)
			raise err
		except KeyboardInterrupt:
			abort()

	def rstdx(self, cmd, remote=None, reuser=None):
		remote = remote if remote else self.remote
		#print(remote)
		reuser = reuser if reuser else self.reuser
		if self.dbg:
			print(bgre(self.stdo))
			print(bgre('  %s %s %s'%(reuser, remote, cmd)))
		ssh = self._ssh_(remote, reuser)
		try:
			_, out, err = ssh.exec_command(cmd)
		except (AttributeError, ssh_exception.SSHException) as err:
			error(self.rstdx, err)
			raise err
		return ''.join(out.readlines()), ''.join(err.readlines())


	def rstdo(self, cmd, remote=None, reuser=None):
		remote = remote if remote else self.remote
		#print(remote)
		reuser = reuser if reuser else self.reuser
		if self.dbg:
			print(bgre(self.rstdo))
			print(bgre('  %s %s %s'%(reuser, remote, cmd)))
		ssh = self._ssh_(remote, reuser)
		try:
			_, out, _ = ssh.exec_command(cmd)
		except (AttributeError, ssh_exception.SSHException) as err:
			error(self.stdo, err)
			raise err
		return ''.join(out.readlines())

	def rstde(self, cmd, remote=None, reuser=None):
		remote = remote if remote else self.remote
		#print(remote)
		reuser = reuser if reuser else self.reuser
		if self.dbg:
			print(bgre(self.stdo))
			print(bgre('  %s %s %s'%(reuser, remote, cmd)))
		ssh = self._ssh_(remote, reuser)
		try:
			_, _, err = ssh.exec_command(cmd)
		except (AttributeError, ssh_exception.SSHException) as err:
			error(self.rstde, err)
			raise err
		return ''.join(err.readlines())

	def rerno(self, cmd, remote=None, reuser=None):
		remote = remote if remote else self.remote
		#print(remote)
		reuser = reuser if reuser else self.reuser
		if self.dbg:
			print(bgre(self.rerno))
			print(bgre('  %s %s %s'%(reuser, remote, cmd)))
		ssh = self._ssh_(remote, reuser)
		try:
			_, out, _ = ssh.exec_command(cmd)
		except (AttributeError, ssh_exception.SSHException) as err:
			error(self.rerno, err)
			raise err
		return int(out.channel.recv_exit_status())

	def roerc(self, cmd, remote=None, reuser=None):
		remote = remote if remote else self.remote
		#print(remote)
		reuser = reuser if reuser else self.reuser
		if self.dbg:
			print(bgre(self.stdo))
			print(bgre('  %s %s %s'%(reuser, remote, cmd)))
		ssh = self._ssh_(remote, reuser)
		try:
			_, out, err = ssh.exec_command(cmd)
		except (AttributeError, ssh_exception.SSHException) as err:
			error(self.roerc, err)
			raise err
		return ''.join(out.readlines()), ''.join(err.readlines()), \
            out.channel.recv_exit_status()

	def get(self, src, trg, remote=None, reuser=None):
		if self.dbg:
			print(bgre(self.get))
		reuser = reuser if reuser else self.reuser
		remote = remote if remote else self.remote
		if not (os.path.isfile(src) or os.path.isfile(trg)):
			raise FileNotFoundError('connot find either %s nor %s'%(src, trg))
		ssh = self._ssh_(remote, reuser)
		scp = ssh.open_sftp()
		return scp.get(src, trg)

	def put(self, src, trg, remote=None, reuser=None):
		if self.dbg:
			print(bgre(self.put))
		reuser = reuser if reuser else self.reuser
		remote = remote if remote else self.remote
		if not (os.path.isfile(src) or os.path.isfile(trg)):
			raise FileNotFoundError('connot find either %s nor %s'%(src, trg))
		ssh = self._ssh_(remote, reuser)
		scp = ssh.open_sftp()
		return scp.put(src, trg)

	def rcompstats(self, src, trg, remote=None, reuser=None):
		if self.dbg:
			print(bgre(self.rcompstats))
		remote = remote if remote else self.remote
		reuser = reuser if reuser else self.reuser
		smt = int(str(int(os.stat(src).st_mtime))[:6])
		rmt = self.rstdo(
            'stat -c %%Y %s'%trg, remote=remote, reuser=user)
		if rmt:
			rmt = int(str(rmt)[:6])
		if rmt == smt:
			return
		srctrg = src, '%s@%s:%s'%(reuser, remote, trg)
		if rmt and int(rmt) > int(smt):
			srctrg = '%s@%s:%s'%(reuser, remote, trg), src
		return srctrg

	def _localstamp(self, trg):
		if self.dbg:
			print(bgre(self._localstamp))
		return int(os.stat(trg).st_atime), int(os.stat(trg).st_mtime)

	def _remotestamp(self, trg, remote, reuser):
		if self.dbg:
			print(bgre(self._remotestamp))
		tat = self.rstdo(
            'stat -c %%X %s'%trg, remote, reuser).strip()
		tmt = self.rstdo(
            'stat -c %%Y %s'%trg, remote, reuser).strip()
		#print(tat)
		if tat and tmt: return int(tat), int(tmt)
		return None, None

	def _setlstamp(self, trg, atime, mtime):
		if self.dbg:
			print(bgre(self._setlstamp))
		os.utime(trg, (atime, mtime))

	def _setrstamp(self, trg, atime, mtime, remote, reuser):
		if self.dbg:
			print(bgre(self._setrstamp))
		self.rstdo(
            'touch -a --date=@%s %s'%(atime, trg), remote, reuser)
		self.rstdo(
            'touch -m --date=@%s %s'%(mtime, trg), remote, reuser)

	def scpcompstats(self, lfile, rfile, remote=None, reuser=None):
		if self.dbg:
			print(bgre(self.scpcompstats))
		reuser = reuser if reuser else self.reuser
		remote = remote if remote else self.remote
		try:
			lat, lmt = self._localstamp(lfile)
			rat, rmt = self._remotestamp(rfile, remote, reuser)
			if rmt == lmt:
				return
			elif rmt and rmt > lmt:
				copy2(lfile, '%s.1'%lfile)
				self.get(rfile, lfile, remote, reuser)
				self._setlstamp(lfile, rat, rmt)
			else:
				self.put(lfile, rfile, remote, reuser)
				self._setlstamp(rfile, lat, lmt)
		except SSHException as err:
			error(err)
		return True



if __name__ == '__main__':
	"""module debugging area"""
	#ssh = SecureSHell(**{'remote':'bigbox.janeiskla.de'})
	#print(ssh.command('cat /etc/debian_version'))



#!/usr/bin/env python3
from re import search

from socket import getfqdn

from netaddr import IPNetwork, IPAddress

def addrmask(address, netmask):
	ip = IPNetwork(str(address)+'/'+str(netmask))
	return str(ip.network), str(ip.prefixlen)

def netips(network):
	ips = []
	for ip in IPNetwork(network):
		ips.append(str(ip))
	if ips != []:
		return ips

def iternet(netaddr, mode=None, verbose=None):
	for ip in netips(netaddr):
		host = getfqdn(ip)
		if mode == 'both':
			ipdns = ip, None
			if host != ip:
				ipdns = ip, host
		elif mode == 'block':
			if host != ip:
				ipdns = ip, host
			else:
				continue
		elif mode == 'avail':
			if host == ip:
				ipdns = ip, None
			else:
				continue
		yield ipdns

def gateway(network=None, ipv4=True, ipv6=False):
	nets = []
	if network:
		ip = IPNetwork(network)
		nets.append(ip.network)
	else:
		for iface in ifaces():
			if iface == 'lo':
				continue
			addrs = ifaddrs(iface, ipv4=ipv4, ipv6=ipv6)
			if not addrs:
				continue
			for ipv in addrs:
				net = '%s/%s' %(addrs[ipv]['addr'], addrs[ipv]['netmask'])
				ip = IPNetwork(net)
				if ip.network:
					nets.append(str(ip.network))
	gates = []
	for net in nets:
		net, last = str(net).split('.')[:-1], str(net).split('.')[-1]
		last = int(last)+1
		if not network or not '/' in network:
			last = 1
		net.append(last)
		gates.append('.'.join(str(n) for n in net))
	return gates


def isip(pattern):
	# return True if "pattern" is RFC conform IP otherwise False
	iplike = r'^(?!0+\.0+\.0+\.0+|255\.255\.255\.255)' \
        r'(25[0-5]|2[0-4]\d|[01]?\d\d?)\.(25[0-5]|2[0-4]\d|[01]?\d\d?)' \
        r'\.(25[0-5]|2[0-4]\d|[01]?\d\d?)\.(25[0-5]|2[0-4]\d|[01]?\d\d?)$'
	if search(iplike, pattern):
		return True
	return False



try:
	from os import getuid
except ImportError:
	def whoami(): return os.environ['USERNAME']

def whoami():
	with open('/etc/passwd', 'r') as pwf:
		pwl = pwf.readlines()
	return [
        u.split(':')[0] for u in pwl if int(u.split(':')[2]) == getuid()][0]
